// Edited 3/17/2015 - 4 space tab spacing

/** \file
	\brief Main file - this is where it all starts, and ends
*/
#include "simpletools.h"
#include "config.h"
#include "dda_queue.h"
#include "dda.h"
#include "gcode_parse.h"
#include "debug.h"
#include "pinio.h"
#include "sergcode.h"
#include "adafruit_ads1015.h"
#include "timer.h"
#include "queue_monitor.h"
#include "time_clock.h"

/// initialise various I/O - set pins as input or output, turn off unused subsystems, etc
void io_init(void)
{
	// setup I/O pins

	// X Stepper
	set_direction(X_STEP_PIN, 1);
	set_output(X_STEP_PIN, 0);

	set_direction(X_DIR_PIN, 1);
	set_output(X_DIR_PIN,  0);	

	#ifdef X_MIN_PIN
		set_direction(X_MIN_PIN, 0);
		set_output(X_MIN_PIN, 0);
	#endif

	#ifdef X_MAX_PIN
		set_direction(X_MAX_PIN, 0);
		set_output(X_MAX_PIN, 0);
	#endif

	// Y Stepper
	set_direction(Y_STEP_PIN, 1);
	set_output(Y_STEP_PIN, 0);

	set_direction(Y_DIR_PIN, 1);
	set_output(Y_DIR_PIN, 0);

	#ifdef Y_MIN_PIN
		set_direction(Y_MIN_PIN, 0);
		set_output(Y_MIN_PIN, 0);
	#endif

	#ifdef Y_MAX_PIN
		set_direction(Y_MAX_PIN, 0);
		set_output(Y_MAX_PIN, 0);
	#endif

	// Z Stepper
	set_direction(Z_STEP_PIN, 1);
	set_output(Z_STEP_PIN, 0);
	
	set_direction(Z_DIR_PIN, 1);
	set_output(Z_DIR_PIN, 0);	

	#ifdef Z_MIN_PIN
		set_direction(Z_MIN_PIN, 0);
		set_output(Z_MIN_PIN, 0);
	#endif

	#ifdef Z_MAX_PIN
		set_direction(Z_MAX_PIN, 0);
		set_output(Z_MAX_PIN, 0);
	#endif

	// E Stepper
	set_direction(E_STEP_PIN, 1);
	set_output(E_STEP_PIN, 0);

	set_direction(E_DIR_PIN, 1);
	set_output(E_DIR_PIN, 0);

	// Common Stepper Enable
	#ifdef STEPPER_ENABLE_PIN

		set_direction(STEPPER_ENABLE_PIN, 1);

		#ifdef STEPPER_INVERT_ENABLE
			set_output(STEPPER_ENABLE_PIN, 0);
		#else
			set_output(STEPPER_ENABLE_PIN, 1);
		#endif
			
	#endif

	// X Stepper Enable
	#ifdef X_ENABLE_PIN

		set_direction(X_ENABLE_PIN, 1);

		#ifdef X_INVERT_ENABLE
			set_output(X_ENABLE_PIN, 0);
		#else
			set_output(X_ENABLE_PIN, 1);
		#endif
		
	#endif

	// Y Stepper Enable
	#ifdef Y_ENABLE_PIN

		set_direction(Y_ENABLE_PIN, 1);

		#ifdef Y_INVERT_ENABLE
			set_output(Y_ENABLE_PIN, 0);
		#else
			set_output(Y_ENABLE_PIN, 1);
		#endif
		
	#endif

	// Z Stepper Enable
	#ifdef Z_ENABLE_PIN

		set_direction(Z_ENABLE_PIN, 1);

		#ifdef Z_INVERT_ENABLE
			set_output(Z_ENABLE_PIN, 0);
		#else
			set_output(Z_ENABLE_PIN, 1);
		#endif
		
	#endif

	// E Stepper Enable
	#ifdef E_ENABLE_PIN

		set_direction(E_ENABLE_PIN, 1);

		#ifdef E_INVERT_ENABLE
			set_output(E_ENABLE_PIN, 0);
		#else
			set_output(E_ENABLE_PIN, 1);
		#endif
		
	#endif

	#ifdef	STEPPER_ENABLE_PIN
		power_off();
	#endif
}

/// Startup code, run when we come out of reset
void init(void)
{
	#ifdef GCODE_SER
 
	// set up serial
	serial_init();

	#endif

	// set up G-code parsing
	gcode_init();

	// set up inputs and outputs
	io_init();

	// set up clock timer
	timer_init();

	// set up queue monitor
	queue_monitor_init();

	// set up dda
	dda_init();

	// set up adc
	adc_init();

	// say hi to host
	//2/25/2015//Bruce//serial_writestr_P(PSTR("start\nok\n"));
}

/// Parse GCODE from an sd card file
void parse_sd(void)
{
	#if defined SD_DO && defined SD_CLK && defined SD_DI && defined SD_CS

	// Mount SD card
	sd_mount(SD_DO, SD_CLK, SD_DI, SD_CS);
    
	FILE * pFile;

	int c;

	pFile = fopen("debug.txt","r");

	if(pFile == NULL)
	{
		print("Error opening file");
	}
	else
	{
		do
		{
			c = getc(pFile);

			// if queue is full, there is no point in reading characters, so wait
			if((c != -1) && (queue_full() == 0))
			{
				gcode_parse_char(c);
				clock_rs();
			}
			else if(c == -1)
			{
				queue_monitor_stop();
				timer_stop(); 
			}
			else
			{
				clock_rs();
			}   			
		}
		while(c != EOF);
        
		fclose (pFile);
	}
    
	#else
    
	print("The proper inputs and outputs must be defined for SD card usage");
    
	#endif
}

/// Parse GCODE from a serial connection
void parse_serial(void)
{
	char c;
 
	while(1)
	{
		c = fdserial_rxChar(sergcode);
    
		// if queue is full, there is no point in reading characters, so wait
		if((c != -1) && (queue_full() == 0))
		{
			gcode_parse_char(c);
			clock_rs();
		}
		else if(c == -1)
		{
			queue_monitor_stop();
			timer_stop(); 
		}
		else
		{
			clock_rs();
		}
	}
}

/// this is where it all starts, and ends
///
/// just run init(), then run an endless loop in either parse_sd() or parse_serial(),
/// where we pass characters from either a serial connection or an sd card file to the 
/// gcode_parse_char() function, meanwhile also checking the clocks

int main(void)
{
//	debug_flags |= DEBUG_PID;
	debug_flags |= DEBUG_DDA;
//	debug_flags |= DEBUG_POSITION;//gcode_process-M114//
	debug_flags |= DEBUG_ECHO;//gcode_parse//

	init();
 
	#ifdef GCODE_SD

	parse_sd();

	#elif defined GCODE_SER

	parse_serial();

	#endif
 
	return 0;
}


/*
int main(void)
{
    adc_init();
    
    //3/11/2015//Bruce//double extruder_fahrenheit;    
    uint8_t extruder_channel = 0;
    //3/11/2015//Bruce//uint8_t bed_channel = 1;    
    
    while(1)
    {
        UpdateTemp(extruder_channel);
        //extruder_fahrenheit = (((extruder_celsius * 9) / 5) + 32);

        print("Current temp: %f C\r\n", extruder_celsius);
        
        pause(500);
    }


    return 0;
}
*/