// Edited 3/17/2015

/** \file
	\brief Sample Configuration

	\this sample contains IO pin setting and is intended to be an example only!
        IO pins MUST be set on a per machine basis!
*/

#ifndef _CONFIG_H
#define _CONFIG_H

#define pgm_read_byte(x) (*((uint8_t *)(x)))
#define pgm_read_word(x) (*((uint16_t *)(x)))

//3/17/2015//Bruce//
// The following define will most likely need work,
// defined the same as pgm_read_word above
#define pgm_read_dword(x) (*((uint16_t *)(x)))

#ifndef BSS
	#define BSS __attribute__ ((__section__ (".bss")))
#endif

/*
	CONTENTS

	1. Mechanical/Hardware
	2. Acceleration settings
	3. Pinouts and defines
	4. Temperature sensors
	5. Heaters
	6. Communication options
	7. Miscellaneous
	8. SD Card Pins
	9. GCODE source
*/

/***************************************************************************\
*                                                                           *
* 1. MECHANICAL/HARDWARE                                                    *
*                                                                           *
\***************************************************************************/

/** \def F_CPU
	CPU clock rate
*/
#ifndef	F_CPU
	#define F_CPU 80000000UL //Forger
#endif

/** \def KINEMATICS

  This defines the type of kinematics your printer uses. That's essential!

  Valid values (see dda_kinematics.h):

  KINEMATICS_STRAIGHT   Motors move axis directions directly. This is the
                        traditional type, found in many printers, including
                        Mendel, Prusa i3, Mendel90, Ormerod, Mantis.

  KINEMATICS_COREXY     A bot using CoreXY kinematics. Typical for CoreXY are
                        long and crossing toothed belts and a print head moving
                        on the X-Y-plane.
*/
#define KINEMATICS KINEMATICS_STRAIGHT

/** \def STEPS_PER_M
	steps per meter ( = steps per mm * 1000 )

	calculate these values appropriate for your machine

	for threaded rods, this is
		(steps motor per turn) / (pitch of the thread) * 1000

	for belts, this is
		(steps per motor turn) / (number of gear teeth) / (belt module) * 1000

	half-stepping doubles the number, quarter stepping requires * 4, etc.

	valid range = 20 to 4'0960'000 (0.02 to 40960 steps/mm)

	all numbers are integers, so no decimal point, please :-)
*/
#define STEPS_PER_M_X 55555 //Forger
#define STEPS_PER_M_Y 55555 //Forger
#define STEPS_PER_M_Z 320000 //Example

/// http://blog.arcol.hu/?p=157 may help with this one
#define STEPS_PER_M_E 320000 //Example


/*
	Values depending on the capabilities of your stepper motors and other mechanics.
		All numbers are integers, no decimals allowed.

		Units are mm/min
*/

/// used for G0 rapid moves and as a cap for all other feedrates
#define MAXIMUM_FEEDRATE_X 200 //Example
#define MAXIMUM_FEEDRATE_Y 200 //Example
#define MAXIMUM_FEEDRATE_Z 100 //Example
#define MAXIMUM_FEEDRATE_E 200 //Example

/// Used when doing precision endstop search and as default feedrate.
#define SEARCH_FEEDRATE_X 50 //Example
#define SEARCH_FEEDRATE_Y 50 //Example
#define SEARCH_FEEDRATE_Z 50 //Example
// no SEARCH_FEEDRATE_E, as E can't be searched

/** \def ENDSTOP_CLEARANCE_X
    \def ENDSTOP_CLEARANCE_Y
    \def ENDSTOP_CLEARANCE_Z

  When hitting an endstop, Teacup properly decelerates instead of doing an
  aprupt stop to save your mechanics. Ineviteably, this means it overshoots
  the endstop trigger point by some distance.

  To deal with this, Teacup adapts homing movement speeds to what your
  endstops can deal with. The higher the allowed acceleration (= deceleration,
  see #define ACCELERATION) and the more clearance the endstop comes with,
  the faster Teacup will do homing movements.

  Set here how many micrometers (mm * 1000) your endstop allows the carriage
  to overshoot the trigger point. Typically 1000 or 2000 for mechanical
  endstops, more for optical ones. You can set it to zero, in which case
  SEARCH_FEEDRATE_{XYZ} is used, but expect very slow homing movements.

  Units: micrometers
  Sane values: 0 to 20000   (0 to 20 mm)
  Valid range: 0 to 1000000
*/
#define ENDSTOP_CLEARANCE_X 1000
#define ENDSTOP_CLEARANCE_Y 1000
#define ENDSTOP_CLEARANCE_Z 100

/**
	Soft axis limits, in mm.
	Define them to your machine's size relative to what your host considers to be the origin.
*/

//#define X_MIN 0.0 //Example
//#define X_MAX 200.0 //Example

//#define Y_MIN 0.0 //Example
//#define Y_MAX 200.0 //Example

//#define Z_MIN 0.0 //Example
//#define Z_MAX 140.0 //Example

/**	\def E_ABSOLUTE
	Some G-Code creators produce relative length commands for the extruder, others absolute ones.
	G-Code using absolute lengths can be recognized when there are G92 E0 commands from time to
	time. If you have G92 E0 in your G-Code, define this flag.

	This is the startup default and can be changed with M82/M83 while running.
*/
// #define E_ABSOLUTE

/***************************************************************************\
*                                                                           *
* 2. ACCELERATION                                                           *
*                                                                           *
* Choose optionally one of ACCELERATION_REPRAP, ACCELERATION_RAMPING or     *
* ACCELERATION_TEMPORAL. With none of them defined, movements are done      *
* without acceleration. Recommended is ACCELERATION_RAMPING.                *
*                                                                           *
* LOOKAHEAD is experimental for now and works in conjunction with           *
* ACCELERATION_RAMPING, only. That's why it's off by default.               *
*                                                                           *
* Also don't forget to adjust ACCELERATION to the capabilities of your      *
* printer. The default is very moderate to be on the safe side.             *
*                                                                           *
\***************************************************************************/

/** \def ACCELERATION_RAMPING
	acceleration and deceleration ramping.
		Each movement starts at (almost) no speed, linearly accelerates to target speed and
		decelerates just in time to smoothly stop at the target.
*/
#define ACCELERATION_RAMPING

/** \def ACCELERATION
	how fast to accelerate when using ACCELERATION_RAMPING.
		given in mm/s^2, decimal allowed, useful range 1. to 10'000. Start with 10. for milling
		(high precision) or 1000. for printing
*/
#define ACCELERATION 1000. //Example

/** \def LOOKAHEAD
  Define this to enable look-ahead during *ramping* acceleration to smoothly
  transition between moves instead of performing a dead stop every move.
  Enabling look-ahead requires about 3600 bytes of flash memory.
*/
//#define LOOKAHEAD

/** \def MAX_JERK_X
    \def MAX_JERK_Y
    \def MAX_JERK_Z
    \def MAX_JERK_E

  When performing look-ahead, we need to decide what an acceptable jerk to the
  mechanics is. Look-ahead attempts to instantly change direction at movement
  crossings, which means instant changes in the speed of the axes participating
  in the movement. Define here how big the speed bumps on each of the axes is
  allowed to be.

  If you want a full stop before and after moving a specific axis, define
  MAX_JERK of this axis to 0. This is often wanted for the Z axis. If you want
  to ignore jerk on an axis, define it to twice the maximum feedrate of this
  axis.

  Having these values too low results in more than neccessary slowdown at
  movement crossings, but is otherwise harmless. Too high values can result
  in stepper motors suddenly stalling. If angles between movements in your
  G-code are small and your printer runs through entire curves full speed,
  there's no point in raising the values.

  Units: mm/min
  Sane values: 0 to 400
  Valid range: 0 to 65535
*/
#define MAX_JERK_X 20 //Example
#define MAX_JERK_Y 20 //Example
#define MAX_JERK_Z 0 //Example
#define MAX_JERK_E 20 //Example

/***************************************************************************\
*                                                                           *
* 3. PINOUTS AND DEFINES                                                    *
*                                                                           *
\***************************************************************************/

/*
	user defined pins
	adjust to suit your electronics,
	or adjust your electronics to suit this
*/
// X Pinouts and defines
//#define X_MIN_PIN 5 //Forger
#define X_DIR_PIN 22 //Forger
#define X_STEP_PIN 23 //Forger
#define X_ENABLE_PIN 24 //Forger
//#define X_MAX_PIN
//#define X_INVERT_DIR
//#define X_INVERT_MIN
//#define X_INVERT_MAX
//#define X_INVERT_ENABLE

// Y Pinouts and defines
//#define Y_MIN_PIN 6 //Forger
#define Y_ENABLE_PIN 25 //Forger
#define Y_STEP_PIN 26 //Forger
#define Y_DIR_PIN 27 //Forger
//#define Y_MAX_PIN
//#define Y_INVERT_DIR
//#define Y_INVERT_MIN
//#define Y_INVERT_MAX
//#define Y_INVERT_ENABLE

// Z Pinouts and defines
//#define Z_MIN_PIN 7 //Forger
#define Z_ENABLE_PIN 0 //Forger
#define Z_STEP_PIN 1 //Forger
#define Z_DIR_PIN 2 //Forger
//#define Z_MAX_PIN
//#define Z_INVERT_DIR
//#define Z_INVERT_MIN
//#define Z_INVERT_MAX
//#define Z_INVERT_ENABLE

// E Pinouts and defines
#define E_DIR_PIN 19 //Forger
#define E_STEP_PIN 20 //Forger
#define E_ENABLE_PIN 21 //Forger
//#define E_INVERT_DIR
//#define E_INVERT_ENABLE

// Miscellaneous pinouts and defines
//#define PS_ON_PIN
//#define PS_MOSFET_PIN
//#define STEPPER_ENABLE_PIN
//#define STEPPER_INVERT_ENABLE

/***************************************************************************\
*                                                                           *
* 6. COMMUNICATION OPTIONS                                                  *
*                                                                           *
\***************************************************************************/

/** \def GCODE_TX
  Serial transmit output pin number.
*/
#define GCODE_TX 30 //Forger

/** \def GCODE_RX
  Serial receive input pin number.
*/
#define GCODE_RX 31 //Forger

/** \def GCODE_MODE
  Set/clear bits to define mode: 
  mode bit 0 = invert rx
  mode bit 1 = invert tx
  mode bit 2 = open-drain/source tx
  mode bit 3 = ignore tx echo on rx
*/
#define GCODE_MODE 0 //Forger

/** \def BAUD
  Rate binary values are transmitted, like 115200, 57600,..., 9600 etc.
*/
#define GCODE_BAUD 115200 //Forger

/** \def I2C_SCL
  Set the SCL pin for I2C communication 
*/
#define I2C_SCL 28

/** \def I2C_SDA
  Set the SDA pin for I2C communication
*/
#define I2C_SDA 29

/** \def I2C_MODE
  Sets the SCL drive mode:
  mode 0 = pulls the line high with pull-up resistor
  mode 1 = drives the line high with setting I/O pin to output.
*/
#define I2C_MODE 0

/***************************************************************************\
*                                                                           *
* 7. MISCELLANEOUS OPTIONS                                                  *
*                                                                           *
\***************************************************************************/

/** \def EECONFIG
  EECONFIG: Enable EEPROM configuration storage.

  Enabled by default. Commenting this out makes the binary several hundred
  bytes smaller, so you might want to disable EEPROM storage on small MCUs,
  like the ATmega168.
*/
//#define EECONFIG

/** \def DEBUG
	DEBUG
		enables /heaps/ of extra output, and some extra M-codes.
		WARNING: this WILL break most host-side talkers that expect particular responses from
		firmware such as reprap host and replicatorG use with serial terminal or other suitable
		talker only.
*/
#define DEBUG

/**
	move buffer size, in number of moves
		note that each move takes a fair chunk of ram (69 bytes as of this writing) so don't make
		the buffer too big - a bigger serial readbuffer may help more than increasing this unless
		your gcodes are more than 70 characters long on average.  However, a larger movebuffer
		will probably help with lots of short consecutive moves, as each move takes a bunch of
		math (hence time) to set up so a longer buffer allows more of the math to be done during
		preceding longer moves
*/
#define MOVEBUFFER_SIZE 6 //Forger

/// this is the scaling of internally stored PID values. 1024L is a good value
#define PID_SCALE 1024L //Forger

/** \def ENDSTOP_STEPS
	number of steps to run into the endstops intentionally
 
	Once in a while, Endstops will trigger a false alarm, Teacup debounces them
	by counting a number of consecutive positives. Valid range is 1...255.
	Use 4 or less for reliable endstops, 8 or even more for flaky ones.
*/
#define ENDSTOP_STEPS 4 //Forger

/***************************************************************************\
*                                                                           *
* 8. SD Card Pins                                                           *
*                                                                           *
* For SD card support, uncomment the following defines and provide the      *
* appropriate pin numbers.  If you are not using an SD card, the following  *
* define items should be commented out.                                     *
*                                                                           *
* Additionally, set the GCODE source location, in the following section.    *
*                                                                           *
\***************************************************************************/

/** \def SD_DI
	SD card DATA IN
*/
#define SD_DI 8 //Forger

/** \def SD_DO
	SD card DATA OUT
*/
#define SD_DO 9 //Forger

/** \def SD_CS
	SD card CHIP SELECT
*/
#define SD_CS 12 //Forger

/** \def SD_CLK
	SD card CLOCK
*/
#define SD_CLK 15 //Forger

/***************************************************************************\
*                                                                           *
* 9. GCODE source                                                           *
*                                                                           *
* Select the appropriate source for the incoming GCODE, either from an      *
* SD card or a serial connection.  This setting should be one or the other, *
* so one will be commented and the other uncommented.                       *
*                                                                           *
\***************************************************************************/

/** \def GCODE_SD
	GCODE will be obtained from an SD card file
*/
#define GCODE_SD //Forger

/** \def GCODE_SER
	GCODE will be obtained from a serial connection
*/
//#define GCODE_SER //Example

/***************************************************************************\
*                                                                           *
* 10. ADC, heating elements, temperature control, etc...                    *
*                                                                           *
* Please refer to the notes associated with the items                       *
*                                                                           *
\***************************************************************************/

// how many sample readings to take from the ADC and average, to determine
// the temperatures read by the thermistors.
#define NUMSAMPLES 16 //Forger

/**
	TEMP_HYSTERESIS: actual temperature must be target +/- hysteresis before
	target 	temperature can be achieved. Unit is degree Celsius.
*/
#define TEMP_HYSTERESIS 5 //Forger

/**
	TEMP_RESIDENCY_TIME: actual temperature must be close to target (within
	set temperature +- TEMP_HYSTERESIS) for this long before target is achieved
	(and a M116 succeeds). Unit is seconds.
*/
#define TEMP_RESIDENCY_TIME 60 //Forger

/** \def BED_HEATER
  Set the BED_HEATER pin - in gcode_process.c (M140)
*/
#define BED_HEATER 16 //Forger

/** \def EXTRUDER_HEATER
  Set the EXTRUDER_HEATER pin - in gcode_process.c (M104)
*/
#define EXTRUDER_HEATER 17 //Forger

/** \def EXTRUDER_FAN
  Set the EXTRUDER_FAN pin - in gcode_process.c (M106)
*/
#define EXTRUDER_FAN 18 //Forger

////////////////////////////////////////////////////////////////

#endif /* _CONFIG_H */