// Edited 3/17/2015 - 4 space tab spacing

/** \file
 \brief I/O primitives - step, enable, direction, endstops etc
*/

#ifndef _PINIO_H
#define _PINIO_H

#include "config.h"
#include "simpletools.h"

/*
Power
*/
/// psu_timeout is set to zero when we step, and increases over time so we can
/// turn the motors off when they've been idle for a while.
/// A second function is to guarantee a minimum on time of the PSU.
/// Timeout counting is done in clock.c.  It is used inside and outside of
/// interrupts, which is why it has been made volatile
extern volatile uint8_t psu_timeout;

static void power_init(void);

inline void power_init(void)
{
	#ifdef PS_MOSFET_PIN

	set_output(PS_MOSFET_PIN, 0);
	set_direction(PS_MOSFET_PIN, 1);

	#endif
}

void power_on(void);
void power_off(void);

///////////////////////////////////////////////////////////////////////////////
/*
X Stepper
*/
#define _x_step(st) set_output(X_STEP_PIN, st)
#define x_step() _x_step(1)

#ifndef X_INVERT_DIR
	#define x_direction(dir) set_output(X_DIR_PIN, dir)
#else
	#define x_direction(dir) set_output(X_DIR_PIN, (dir)^1)
#endif

#ifdef X_MIN_PIN
	#ifndef X_INVERT_MIN
		#define x_min() (get_output(X_MIN_PIN)?1:0)
	#else
		#define x_min() (get_output(X_MIN_PIN)?0:1)
	#endif
#else
	#define x_min() (0)
#endif

#ifdef X_MAX_PIN
	#ifndef X_INVERT_MAX
		#define x_max() (get_output(X_MAX_PIN)?1:0)
	#else
		#define x_max() (get_output(X_MAX_PIN)?0:1)
	#endif
#else
	#define x_max() (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
Y Stepper
*/
#define _y_step(st) set_output(Y_STEP_PIN, st)
#define y_step() _y_step(1)

#ifndef Y_INVERT_DIR
	#define y_direction(dir) set_output(Y_DIR_PIN, dir)
#else
	#define y_direction(dir) set_output(Y_DIR_PIN, (dir)^1)
#endif

#ifdef Y_MIN_PIN
	#ifndef Y_INVERT_MIN
		#define y_min() (get_output(Y_MIN_PIN)?1:0)
	#else
		#define y_min() (get_output(Y_MIN_PIN)?0:1)
	#endif
#else
	#define y_min() (0)
#endif

#ifdef Y_MAX_PIN
	#ifndef Y_INVERT_MAX
		#define y_max() (get_output(Y_MAX_PIN)?1:0)
	#else
		#define y_max() (get_output(Y_MAX_PIN)?0:1)
	#endif
#else
	#define y_max() (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
Z Stepper
*/
#if defined Z_STEP_PIN && defined Z_DIR_PIN
	#define _z_step(st) set_output(Z_STEP_PIN, st)
	#define z_step() _z_step(1)

	#ifndef Z_INVERT_DIR
		#define z_direction(dir) set_output(Z_DIR_PIN, dir)
	#else
		#define z_direction(dir) set_output(Z_DIR_PIN, (dir)^1)
	#endif
#else
	#define _z_step(x) do {} while (0)
	#define z_step() do {} while (0)
	#define z_direction(x) do {} while (0)
#endif

#ifdef Z_MIN_PIN
	#ifndef Z_INVERT_MIN
		#define z_min() (get_output(Z_MIN_PIN)?1:0)
	#else
		#define z_min() (get_output(Z_MIN_PIN)?0:1)
	#endif
#else
	#define z_min() (0)
#endif

#ifdef Z_MAX_PIN
	#ifndef Z_INVERT_MAX
		#define z_max() (get_output(Z_MAX_PIN)?1:0)
	#else
		#define z_max() (get_output(Z_MAX_PIN)?0:1)
	#endif
#else
	#define z_max() (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
E Stepper
*/
#if defined E_STEP_PIN && defined E_DIR_PIN
	#define _e_step(st) set_output(E_STEP_PIN, st)

	#define e_step() _e_step(1)

	#ifndef E_INVERT_DIR
		#define e_direction(dir) set_output(E_DIR_PIN, dir)
	#else
		#define e_direction(dir) set_output(E_DIR_PIN, (dir)^1)
	#endif
#else
	#define _e_step(st) do {} while (0)
	#define e_step() do {} while (0)
	#define e_direction(dir) do {} while (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
End Step - All Steppers
(so we don't have to delay in interrupt context)
*/
#define unstep() do { _x_step(0); _y_step(0); _z_step(0); _e_step(0); } while (0)

/*
Stepper Enable Pins
*/
#ifdef STEPPER_ENABLE_PIN
	#ifdef STEPPER_INVERT_ENABLE
		#define stepper_enable() set_output(STEPPER_ENABLE_PIN, 0)
		#define stepper_disable() set_output(STEPPER_ENABLE_PIN, 1)
	#else
		#define stepper_enable() set_output(STEPPER_ENABLE_PIN, 1)
		#define stepper_disable() set_output(STEPPER_ENABLE_PIN, 0)
	#endif
#else
	#define stepper_enable() do {} while (0)
	#define stepper_disable() do {} while (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
X Stepper Enable
*/
#ifdef X_ENABLE_PIN
	#ifdef X_INVERT_ENABLE
		#define x_enable() set_output(X_ENABLE_PIN, 0)
		#define x_disable() set_output(X_ENABLE_PIN, 1)
	#else
		#define x_enable() set_output(X_ENABLE_PIN, 1)
		#define x_disable() set_output(X_ENABLE_PIN, 0)
	#endif
#else
	#define x_enable() do {} while (0)
	#define x_disable() do {} while (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
Y Stepper Enable
*/
#ifdef Y_ENABLE_PIN
	#ifdef Y_INVERT_ENABLE
		#define y_enable() set_output(Y_ENABLE_PIN, 0)
		#define y_disable() set_output(Y_ENABLE_PIN, 1)
	#else
		#define y_enable() set_output(Y_ENABLE_PIN, 1)
		#define y_disable() set_output(Y_ENABLE_PIN, 0)
	#endif
#else
	#define y_enable() do {} while (0)
	#define y_disable() do {} while (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
Z Stepper Enable
*/
#ifdef Z_ENABLE_PIN
	#ifdef Z_INVERT_ENABLE
		#define z_enable() set_output(Z_ENABLE_PIN, 0)
		#define z_disable() set_output(Z_ENABLE_PIN, 1)
	#else
		#define z_enable() set_output(Z_ENABLE_PIN, 1)
		#define z_disable() set_output(Z_ENABLE_PIN, 0)
	#endif
#else
	#define z_enable() do { } while (0)
	#define z_disable() do { } while (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
E Stepper Enable
*/
#ifdef E_ENABLE_PIN
	#ifdef E_INVERT_ENABLE
		#define e_enable() set_output(E_ENABLE_PIN, 0)
		#define e_disable() set_output(E_ENABLE_PIN, 1)
	#else
		#define e_enable() set_output(E_ENABLE_PIN, 1)
		#define e_disable() set_output(E_ENABLE_PIN, 0)
	#endif
#else
	#define e_enable() do {} while (0)
	#define e_disable() do {} while (0)
#endif
///////////////////////////////////////////////////////////////////////////////
/*
Endstops
*/
static void endstops_on(void) __attribute__ ((always_inline));

inline void endstops_on(void)
{
	#ifdef X_MIN_PIN
		set_output(X_MIN_PIN, 1);
	#endif

	#ifdef X_MAX_PIN
		set_output(X_MAX_PIN, 1);
	#endif

	#ifdef Y_MIN_PIN
		set_output(Y_MIN_PIN, 1);
	#endif

	#ifdef Y_MAX_PIN
		set_output(Y_MAX_PIN, 1);
	#endif

	#ifdef Z_MIN_PIN
		set_output(Z_MIN_PIN, 1);
	#endif

	#ifdef Z_MAX_PIN
		set_output(Z_MAX_PIN, 1);
	#endif
}

static void endstops_off(void) __attribute__ ((always_inline));

inline void endstops_off(void)
{
	#ifdef X_MIN_PIN
		set_output(X_MIN_PIN, 0);
	#endif

	#ifdef X_MAX_PIN
		set_output(X_MAX_PIN, 0);
	#endif

	#ifdef Y_MIN_PIN
		set_output(Y_MIN_PIN, 0);
	#endif

	#ifdef Y_MAX_PIN
		set_output(Y_MAX_PIN, 0);
	#endif

	#ifdef Z_MIN_PIN
		set_output(Z_MIN_PIN, 0);
	#endif

	#ifdef Z_MAX_PIN
		set_output(Z_MAX_PIN, 0);
	#endif
}
///////////////////////////////////////////////////////////////////////////////

#endif /* _PINIO_H */